/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "edit.h"

// Update the TextBuffer that is sent to this function. It marks it modified
// when text has been entered and shows the cursor on screen.
void OpenLDev::openldev_update_textview (GtkSourceBuffer *buffer)
{
  noteman->noteman_mark_current_tab_modified(true); 
  noteman->noteman_scroll_to_mark();
}

// Cut the currently selected text from the buffer. Then add the text to the
// actionmanager of the current buffer and redo the syntax highlighting
void openldev_cut (GtkMenuItem *menuitem,
                   gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkClipboard *clipboard = gtk_clipboard_get(GDK_NONE);
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
  
    gtk_text_buffer_cut_clipboard(GTK_TEXT_BUFFER(buffer), clipboard, true);
    ptr->openldev_update_textview(buffer);
  }
}

// Cut the currently selected text from the buffer.
void openldev_copy (GtkMenuItem *menuitem,
                    gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkClipboard *clipboard = gtk_clipboard_get(GDK_NONE);
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
    
    gtk_text_buffer_copy_clipboard(GTK_TEXT_BUFFER(buffer), clipboard);
  }
}

// Paste the clipboard text to the current cursor position. Then add the text 
// to the actionmanager of the current buffer and redo the syntax highlighting
void openldev_paste (GtkMenuItem *menuitem,
                     gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkClipboard *clipboard = gtk_clipboard_get(GDK_NONE);
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
    
    gtk_text_buffer_paste_clipboard (GTK_TEXT_BUFFER(buffer), 
                                     clipboard, NULL, true);
    ptr->openldev_update_textview(buffer);
  }
}

// Select all text in the text_view
void openldev_select_all (GtkMenuItem *menuitem,
                          gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkTextIter iter;
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
    gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &iter);
    gtk_text_buffer_move_mark(GTK_TEXT_BUFFER(buffer), 
                              gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)),
                              &iter);
    gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer), &iter);
    gtk_text_buffer_move_mark(GTK_TEXT_BUFFER(buffer),
                              gtk_text_buffer_get_selection_bound(GTK_TEXT_BUFFER(buffer)),
                              &iter);
  }
}

// Deselect any text that is selected
void openldev_deselect (GtkMenuItem *menuitem,
                        gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
    GtkTextMark *mark = gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer));
    GtkTextIter iter;
    
    gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(buffer), &iter, mark);
    gtk_text_buffer_move_mark(GTK_TEXT_BUFFER(buffer),
                              gtk_text_buffer_get_selection_bound(GTK_TEXT_BUFFER(buffer)),
                              &iter);
  }
}

// Undo the last action that was performed. If there was no action, then don't
// do anything. To undo a delete, just insert the text. To undo an insert, the
// text must be removed from the specified offset & then backwards the length
void openldev_undo (GtkMenuItem *menuitem,
                    gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
    if (gtk_source_buffer_can_undo(ptr->noteman->noteman_get_textbuffer()))
      gtk_source_buffer_undo(ptr->noteman->noteman_get_textbuffer());
}

// Redo the last action that was undone. If there was no action, then don't
// do anything. To redo an insert, you must remove the text from the specified
// offset to backwards the length of the text. To redo a delete, just move
// backward the length of the string & insert it
void openldev_redo (GtkMenuItem *menuitem,
                    gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
    if (gtk_source_buffer_can_redo(ptr->noteman->noteman_get_textbuffer()))
      gtk_source_buffer_redo(ptr->noteman->noteman_get_textbuffer());
}
