/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "fileio.h"

// Create a new project by calling the new project dialog & then open the
// project that was created.
void openldev_create_new_project (GtkMenuItem* menuitem, 
                                  gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  NewProjectDialog *dialog;
  dialog = new NewProjectDialog(ptr->list_of_templates);
  int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));

  // Open newly created project if OK is pressed
  switch (result)
  {
  case GTK_RESPONSE_ACCEPT:
    ptr->openldev_open_project(dialog->get_filename());
    openldev_autogen (menuitem, data);
    break;
  }
}

// Create a new file if a project is currently open. Create it by calling the
// new file dialog & then open the file that was created.
void openldev_create_new_file (GtkMenuItem* menuitem, 
                               gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
  {
    error("Please open or create\na project first!");
  } 
  // Only create new file if project is open
  else 
  {
    NewFileDialog *dialog;
    dialog = new NewFileDialog (ptr->project);
    int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));

    // Open the newly created file
    switch (result)
    {
    case GTK_RESPONSE_ACCEPT:
      if (dialog->get_add_to_project())
      {
        string fn = dialog->get_filename();
        while (fn.find("/", 0) != string::npos)
          fn.erase(0, fn.find("/", 0)+1);        
        fn += "/";
        
        ptr->project->add_file(fn);
        ptr->openldev_populate_treeview();
        ptr->project->write_project_file();
        ptr->project->write_source_makefile();
      }
      
      ptr->openldev_open_file(dialog->get_filename());
      break;
    }
  }
}

// Process the File->Open command by creating a FileChooserDialog. Upon getting
// the filename, check whether the file is a project file. If it is a project
// file, open the project. Otherwise, call the function to open the file.
void openldev_open_file_or_project (GtkMenuItem* menuitem, 
                                    gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;

  // Create file selector
  GtkWidget *dialog;
  dialog  = gtk_file_chooser_dialog_new("Choose a project or File", 
                                        GTK_WINDOW(ptr->window),
                                        GTK_FILE_CHOOSER_ACTION_OPEN, 
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_OPEN, GTK_RESPONSE_OK,
                                        NULL);
  
  // Set default folder
  if (!ptr->project->get_name().empty())
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         ptr->project->get_location().c_str());
  else
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         g_get_home_dir());  
  
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .odp or .openldev file
      string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog));
      if (filename.find(".odp", 0) != string::npos || 
          filename.find(".openldev", 0) != string::npos)
        ptr->openldev_open_project(filename);
      else
        ptr->openldev_open_file(filename);
      break;
    }
    default:
      break;
  }
  
  gtk_widget_hide(dialog);
}

// Open the specified file if it exists. This function receives the filename that
// it is supposed to open. If a project is open, the file is added as the
// last opened file. The file is then opened, syntax highlighting is applied, 
// line numbering is calculated and displayed, signals are connected to the
// keystroke action and final initialization is applied
void OpenLDev::openldev_open_file (string filename)
{
  if (!g_file_test (filename.c_str(), G_FILE_TEST_EXISTS))
    error("Cannot open file. It does not exist!");
  else
  {
    // Add a new tab to the notebookmanager
    if (noteman->noteman_add_tab(filename, env))
    { 
      string text, title, cliptext;
  
      // Change last file open in project
      if (!project->get_name().empty()) 
      {
        string fn = filename;
        int location = fn.find('/', 0);
        while (location != (signed int)string::npos) 
        {
          fn.erase(0,location+1);
          location = fn.find('/', 0);
        }
        if (project->get_source_files().find(fn, 0) != string::npos ||
            project->get_header_files().find(fn, 0) != string::npos) 
        {
          project->set_lastfile(fn);
          project->write_project_file();
        }
      }
  
      // Open file & read in text
      text = read_file(noteman->noteman_get_current_filename());
      
      // Add text to text buffer after applying syntax highlighting
      GtkSourceBuffer *buffer = noteman->noteman_get_textbuffer();
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), text.c_str(), -1);
      
      // Set the text to the buffer, connect keystroke function & show/hide line numbering
      gtk_text_view_set_buffer(GTK_TEXT_VIEW(noteman->noteman_get_textview()),
                               GTK_TEXT_BUFFER(buffer));
      
      g_signal_connect (G_OBJECT(noteman->noteman_get_textbuffer()), "mark-set",
                        G_CALLBACK(on_status_bar_update), (gpointer)this);
      g_signal_connect (G_OBJECT(noteman->noteman_get_textview()), "key-press-event",
                        G_CALLBACK(on_textview_handle_autocomplete), (gpointer)this);
      noteman->noteman_set_properties(false, env);
      
      string font_description = env->get_font_face() + " " + env->get_font_size();
      PangoFontDescription *fd = pango_font_description_from_string(font_description.c_str());
      noteman->noteman_modify_fonts(fd);
    }
  }
}

// The project specified is opened. This function receives the filename of the
// project file. If the project was loaded, the last file opened is reloaded,
// the filelist is populated and the last project open is set to this project
void OpenLDev::openldev_open_project(string filename)
{  
  noteman->noteman_erase_all_tabs();
  bool success = project->load_project_file(filename);
 
  // If the project was opened
  if (success)
  {
    // Open the last file if it exists
    ptemplate->load_template(project->get_template_name());
    if (!project->get_lastfile().empty())
    {
      string fn = string(project->get_location()) + "src/" 
        + string(project->get_lastfile());
      openldev_open_file(fn);
    }

    // Make environment ready for project
    openldev_populate_treeview();
	  env->set_lastproject(filename);
    env->write_settings_file();
    
    string fns = project->get_source_files() + project->get_header_files(),
           loc = project->get_location() + string("src/");
    noteman->noteman_autolist_clean_all ();
    noteman->noteman_autolist_parse_files (fns, loc);
  }
}

// Save the current file & mark the file as unmodified. If the file does not
// have a name, then save as is called
void openldev_save (GtkMenuItem* menuitem, 
                    gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    if (ptr->noteman->noteman_get_current_filename().empty() || 
        ptr->noteman->noteman_get_current_filename() == "EMPTY")
      openldev_save_as (menuitem, data);
  
    // Save the file & set as unmodified
    GtkTextIter start, end;
    GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
    
    gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER(buffer), &start);
    gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(buffer), &end);
    write_file(ptr->noteman->noteman_get_current_filename(), 
      gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &start, &end, true));
    
    ptr->noteman->noteman_mark_current_tab_modified(false);
    string files = ptr->noteman->noteman_get_current_short_filename(),
           loc = ptr->project->get_location() + string("src/");
    
    if (ptr->project->get_file_in_project(ptr->noteman->noteman_get_current_short_filename()))
      ptr->noteman->noteman_autolist_parse_file (files, loc);
  }
}

// Save the current file by a different name. The FileChooserDialog allows the
// user to choose the location and filename. Note: This function does not actually
// save the file, but rather just chooses a new name for it & calls on_file_save
void openldev_save_as (GtkMenuItem* menuitem, 
                       gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {
    GtkWidget *dialog;
    dialog = gtk_file_chooser_dialog_new ("Save file as", 
                                          GTK_WINDOW(ptr->window),
                                          GTK_FILE_CHOOSER_ACTION_SAVE,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                          NULL);
  
    // Set default folder
    string folder_name = string(ptr->project->get_location()) + "src/";
    
    if (!ptr->project->get_name().empty())
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           folder_name.c_str());  
    else
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           g_get_home_dir()); 
  
    // Add the filename to the text entry in SaveAs dialog
    if (ptr->noteman->noteman_get_current_filename().empty() || 
        ptr->noteman->noteman_get_current_filename() == "EMPTY")
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), "untitled");
    else
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog),
          ptr->noteman->noteman_get_current_short_filename().c_str());
  
    // Open fileselection dialog to get file location
    int result = gtk_dialog_run (GTK_DIALOG(dialog));
    switch (result)
    {
      // If OK is pressed
      case (GTK_RESPONSE_OK):
      {
        ptr->noteman->noteman_set_filename(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));  
        openldev_save (menuitem, data);
        break;
      }
      default:
        break;
    }
    
    gtk_widget_hide(dialog);
  }
}

// Call the notebookmanager function to save all of the files that are
// currently open
void openldev_save_all (GtkMenuItem* menuitem, 
                        gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  if (ptr->noteman->noteman_get_file_open())
    ptr->noteman->noteman_save_all_files();
}

// If the user chooses to allow it, revert to the saved version of a file. Then
// the syntax highlighting, action manager and line numbering are refreshed
void openldev_reload_saved (GtkMenuItem* menuitem, 
                            gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  if (ptr->noteman->noteman_get_file_open())
    if (question("Are you sure you want to reload this file?\nAny unsaved changes will be lost."))
      ptr->noteman->noteman_revert_to_saved();
}
