/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "menu.h"

#define NUM_ENTRIES 46
static GtkActionEntry entries[] = {
  { "File", NULL, "_File" },
  { "Edit", NULL, "_Edit" },
  { "View", NULL, "_View" },
  { "Project", NULL, "_Project" },
  { "Build", NULL, "_Build" },
  { "Help", NULL, "_Help" },
  
  { "FileNewFile", GTK_STOCK_NEW, "_New File", "<control>n", "Create a new file", G_CALLBACK(openldev_create_new_file) },
  { "FileNewProject", NULL, "New Project", NULL, "Create a new project", G_CALLBACK(openldev_create_new_project) },
  { "FileOpen", GTK_STOCK_OPEN, "_Open", "<control>o", "Open an existing file", G_CALLBACK(openldev_open_file_or_project) },
  { "FileOpen2", GTK_STOCK_OPEN, "_Open File", NULL, "Open an existing file", G_CALLBACK(openldev_open_file_or_project) },
  { "FileSave", GTK_STOCK_SAVE, "_Save", "<control>s", "Save current file", G_CALLBACK(openldev_save) },
  { "FileSaveAs", GTK_STOCK_SAVE_AS, "Save _As", NULL, "Save file by new name", G_CALLBACK(openldev_save_as) },
  { "FileSaveAll", NULL, "Save All", NULL, "Save all open files", G_CALLBACK(openldev_save_all) },
  { "FileRevert", GTK_STOCK_REVERT_TO_SAVED, "_Reload File", NULL, "Revert to saved file", G_CALLBACK(openldev_reload_saved) },
  { "FilePrint", GTK_STOCK_PRINT, "_Print", "<control>p", "Print the current file", G_CALLBACK(openldev_print_file) },
  { "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, "Print Preview", "<control><shift>p", "Preview the current file", G_CALLBACK(openldev_print_preview) },
  { "FileClose", GTK_STOCK_CLOSE, "_Close Project", NULL, "Close current project", G_CALLBACK(openldev_close_project) },
  { "FileQuit", GTK_STOCK_QUIT, "_Quit", "<control>q", "Quit the application", G_CALLBACK(openldev_quit) },

  { "EditUndo", GTK_STOCK_UNDO, "_Undo", "<control>z", "Undo last action", G_CALLBACK(openldev_undo) },
  { "EditRedo", GTK_STOCK_REDO, "_Redo", "<control>y", "Redo last undone action", G_CALLBACK(openldev_redo) },
  { "EditCut", GTK_STOCK_CUT, "C_ut", "<control>x", "Cut selected text", G_CALLBACK(openldev_cut) },
  { "EditCopy", GTK_STOCK_COPY, "_Copy", "<control>c", "Copy selected text", G_CALLBACK(openldev_copy) },
  { "EditPaste", GTK_STOCK_PASTE, "_Paste", "<control>v", "Paste text from clipboard", G_CALLBACK(openldev_paste) },
  { "EditSelectAll", NULL, "Select _All", "<control>a", "Select all text", G_CALLBACK(openldev_select_all) },
  { "EditDeselect", NULL, "_Deselect", NULL, "Deselect selected text", G_CALLBACK(openldev_deselect) },
  { "EditFind", GTK_STOCK_FIND, "_Find", "<control>f", "Open the Find toolbar", G_CALLBACK(openldev_find) },
  { "EditFindNext", GTK_STOCK_FIND, "Find _Next", "F3", "Find next", G_CALLBACK(openldev_find_next) },
  { "EditGotoLine", GTK_STOCK_JUMP_TO, "_Goto Line", "<control>g", "Open Goto Line toolbar", G_CALLBACK(openldev_goto_line) },
  { "EditSettings", GTK_STOCK_PREFERENCES, "Preferences", NULL, "Edit Environment settings", G_CALLBACK(openldev_environment_settings) },
  
  { "ViewFiles", GTK_STOCK_JUSTIFY_LEFT, "_Project Files", NULL, "Show or hide project files", G_CALLBACK(openldev_view_fileviewer) },
  { "ViewOutput", GTK_STOCK_JUSTIFY_LEFT, "_Build Output", NULL, "Show or hide build output", G_CALLBACK(openldev_view_buildbox) },
  
  { "ProjectAdd", GTK_STOCK_ADD, "_Add Files", NULL, "Add files to project", G_CALLBACK(openldev_add_files) },
  { "ProjectRemove", GTK_STOCK_REMOVE, "_Remove File", NULL, "Remove selected file from project", G_CALLBACK(openldev_remove_file) },
  { "ProjectConfigure", GTK_STOCK_PREFERENCES, "_Configure Project", NULL, "Edit project properties", G_CALLBACK(openldev_project_properties) },
  { "ProjectInfo", GTK_STOCK_PROPERTIES, "_Project Info", NULL, "View project info", G_CALLBACK(openldev_project_info) },
  
  { "BuildCompile", "COMPILE", "_Compile", "F9", "Compile current file", G_CALLBACK(openldev_compile) },
  { "BuildBuild", "BUILD_SM", "_Build", "F11", "Build project tree", G_CALLBACK(openldev_build) },
  { "BuildBuild2", "BUILD_LG", "_Build", NULL, "Build project tree", G_CALLBACK(openldev_build) },
  { "BuildExecute", GTK_STOCK_EXECUTE, "_Execute", "F4", "Execute project", G_CALLBACK(openldev_execute) },
  { "BuildAutogen", GTK_STOCK_CONVERT, "_Auto Generate", NULL, "Auto generate and configure", G_CALLBACK(openldev_autogen) },
  { "BuildConfigure", NULL, "Con_figure", NULL, "Run configure script", G_CALLBACK(openldev_configure) },
  { "BuildClean", GTK_STOCK_CLEAR, "_Make Clean", NULL, "Clean distribution tree", G_CALLBACK(openldev_clean) },
  { "BuildStop", GTK_STOCK_STOP, "_Stop", "F12", "Stop current action", G_CALLBACK(openldev_stop) },
  
  { "HelpDocs", GTK_STOCK_HELP, "_Documentation", "F1", "View project documentation", G_CALLBACK(openldev_help) },
  { "HelpBugs", NULL, "_Report Bugs", NULL, "Report bugs", G_CALLBACK(openldev_report_bug) },
  { "HelpAbout", GTK_STOCK_ABOUT, "_About", "<alt>a", "About OpenLDev", G_CALLBACK(openldev_about) }
};

#define NUM_ENTRIES_TOGGLE 3
static GtkToggleActionEntry toggle_entries[] = {
  { "ViewToolbar", NULL, "_Toolbar", NULL, "Show or hide toolbar", G_CALLBACK(openldev_view_toolbar), true },
  { "ViewStatusBar", NULL, "_Status Bar", NULL, "Show or hide status bar", G_CALLBACK(openldev_view_statusbar), true },
  { "ViewLineNumbering", NULL, "_Line Numbering", NULL, "Show or hide line numbers", G_CALLBACK(openldev_view_line_numbers), true },
};

// This function creates a new icon factory that adds the stock menu/toolbar
// items for the compile, build, project files and about commands
void OpenLDev::openldev_create_icon_factory()
{
  GtkIconFactory *icon_factory = gtk_icon_factory_new();;
  
  // Add new stock menu items
  add_new_stock_item(icon_factory, "Compile", "COMPILE", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/compile_sm.png");
  add_new_stock_item(icon_factory, "Build", "BUILD_SM", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/build_sm.png");
  add_new_stock_item(icon_factory, "Build", "BUILD_LG", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/build_lg.png");
  
  // Add items to default
  gtk_icon_factory_add_default (icon_factory);
}

void OpenLDev::openldev_create_menus_and_toolbar()
{
  // Create menu actions and icons
  openldev_create_icon_factory();
  
  GtkActionGroup *actgroup = gtk_action_group_new ("ActionGroup");
  gtk_action_group_add_actions (actgroup, entries, NUM_ENTRIES, (gpointer)this);
  gtk_action_group_add_toggle_actions (actgroup, toggle_entries, NUM_ENTRIES_TOGGLE, (gpointer)this);
  
  string ui_file = string(INSTALL_PREFIX) + "/share/openldev/menus/mainmenu.ui",
         ui_file2 = string(INSTALL_PREFIX) + "/share/openldev/menus/toolbar.ui",
         ui_file3 = string(INSTALL_PREFIX) + "/share/openldev/menus/popup_treeview.ui";
  GError *uierror;
  GtkUIManager *uimanager = gtk_ui_manager_new ();
  
  if (!gtk_ui_manager_add_ui_from_file (uimanager, ui_file.c_str(), &uierror))
    error ("Cannot load file: mainmenu.ui");
  if (!gtk_ui_manager_add_ui_from_file (uimanager, ui_file2.c_str(), &uierror))
    error ("Cannot load file: toolbar.ui");
  if (!gtk_ui_manager_add_ui_from_file (uimanager, ui_file3.c_str(), &uierror))
    error ("Cannot load file: popup_treeview.ui");
  
  gtk_ui_manager_insert_action_group (uimanager, actgroup, 0);
  gtk_window_add_accel_group (GTK_WINDOW(window), 
                              gtk_ui_manager_get_accel_group(uimanager));
  
  menubar = gtk_ui_manager_get_widget (uimanager, "/MenuBar");
  toolbar = gtk_ui_manager_get_widget (uimanager, "/ToolBar");
  menu_source_browser = gtk_ui_manager_get_widget (uimanager, "/TreeViewMenu");
  
  gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
  gtk_toolbar_set_show_arrow (GTK_TOOLBAR(toolbar), false);
}
