/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef NOTEBOOKMANAGER_H
#define NOTEBOOKMANAGER_H

#include "utils.h"
#include "notebooktab.h"
#include "autolist.h"

#include <libgnomevfs/gnome-vfs.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <vector>
using namespace std;

struct AutoComplete
{
  GtkWidget *autocomplete, *autocompletewin;
};

// Notebook manager class that handles the tabs
class NotebookManager
{
public:
	NotebookManager ();

  // Functions to get private data and widgets
	GtkWidget* noteman_get_notebook ();
  GtkWidget* noteman_get_textview ();
  GtkSourceBuffer* noteman_get_textbuffer ();
  string noteman_get_current_filename ();
  string noteman_get_current_short_filename ();
  int noteman_get_current_tab ();
  int noteman_get_insert_offset ();
  int noteman_get_selection_offset ();
  bool noteman_get_file_open ();
  
  // Functions to set private data and widgets
  void noteman_set_filename (string);
  void noteman_set_properties (bool, EnvironmentSettings*);
  void noteman_set_properties_one_tab (int, EnvironmentSettings*);

  // Tab management functions
	bool noteman_add_tab (string, EnvironmentSettings*);
  void noteman_erase_all_tabs ();
  void noteman_save_all_files ();
  void noteman_mark_current_tab_modified (bool);
  
  // Textview modification functions
  void noteman_scroll_to_mark ();
  void noteman_modify_fonts (PangoFontDescription*);
  void noteman_revert_to_saved ();
  
  void noteman_autolist_parse_files (string, string);
  void noteman_autolist_parse_file (string, string);
  void noteman_autolist_clean_all ();

private:
  friend void on_notebook_event (GtkNotebook*, gint, gpointer);
  friend void on_delete_tab (GtkWidget*, gpointer);
  friend bool on_textview_keypress (GtkWidget*, GdkEvent*, gpointer);
  friend bool on_textview_focus_out (GtkWidget*, GdkEventFocus*, gpointer);

  friend void autocomplete_show_popup (gpointer, gchar);
  friend void autocomplete_hide_popup (gpointer);
  friend void autocomplete_populate (gpointer, vector<string>);
  friend void autocomplete_position (gpointer, int, int);
  friend void autocomplete_scroll_view (gpointer, guint);
  friend bool autocomplete_insert_word (gpointer);
	
private:
  GtkWidget *notebook;
	int tab_count;
	vector<NotebookTab> tabs;
  AutoComplete ac;
  AutoList al;
};

#endif
