/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GTKMM_MAIN_H
#define GTKMM_MAIN_H

// The motherload of includes for all other files. (Note: This file is included
// by most others in this code, so most Gtk files will already be included)
#include "editor.h"
#include "notebookmanager.h"
#include "project.h"
#include "template.h"
#include "variables.h"
#include "outputmanager.h"
#include "findtoolbar.h"
#include "gototoolbar.h"
#include "browser.h"

#include <sys/stat.h>

const char *const clipboard = "";

// Main class for openldev
class OpenLDev
{
public:
  OpenLDev (GtkWidget*);                         

  void openldev_open_file (string);
  void openldev_open_project (string);
  void openldev_handle_treeview_selection();

  void openldev_mark_found_text (GtkTextIter*, GtkTextIter*);
  void openldev_update_textview (GtkSourceBuffer*);
  
  // Initialization functions
  void openldev_populate_window();
  void openldev_initialize_objects();
  void openldev_populate_treeview();
  void openldev_create_find_toolbar();
  void openldev_create_goto_line_toolbar();
  void openldev_create_icon_factory();
  void openldev_connect_signals();
  void openldev_create_menus_and_toolbar();
  void openldev_initialize_treeview();

public:
  EnvironmentSettings *env;
  NotebookManager *noteman;
  ProjectSettings *project;

private:
  // File functions
  friend void openldev_create_new_project (GtkMenuItem*, gpointer);  
  friend void openldev_create_new_file (GtkMenuItem*, gpointer);
  friend void openldev_quit (GtkMenuItem*, gpointer);
  friend void openldev_open_file_or_project (GtkMenuItem*, gpointer);                
  friend void openldev_save (GtkMenuItem*, gpointer);                
  friend void openldev_save_as (GtkMenuItem*, gpointer);
  friend void openldev_save_all (GtkMenuItem*, gpointer);
  friend void openldev_print_file (GtkMenuItem*, gpointer);
  friend void openldev_print_preview (GtkMenuItem*, gpointer);
  friend void openldev_reload_saved (GtkMenuItem*, gpointer);
  friend void openldev_close_project (GtkMenuItem*, gpointer);

  // Edit commands
  friend void openldev_find_next (GtkMenuItem*, gpointer);
  friend void openldev_undo (GtkMenuItem*, gpointer);
  friend void openldev_redo (GtkMenuItem*, gpointer);
  friend void openldev_cut (GtkMenuItem*, gpointer);
  friend void openldev_copy (GtkMenuItem*, gpointer);
  friend void openldev_paste (GtkMenuItem*, gpointer);
  friend void openldev_select_all (GtkMenuItem*, gpointer);
  friend void openldev_deselect (GtkMenuItem*, gpointer);
  friend void openldev_find (GtkMenuItem*, gpointer);
  friend void openldev_goto_line (GtkMenuItem*, gpointer);
  friend void openldev_environment_settings (GtkMenuItem*, gpointer);
  
  // View commands
  friend void openldev_view_toolbar (GtkMenuItem*, gpointer);
  friend void openldev_view_statusbar (GtkMenuItem*, gpointer);
  friend void openldev_view_fileviewer (GtkMenuItem*, gpointer);
  friend void openldev_view_buildbox (GtkMenuItem*, gpointer);
  friend void openldev_view_line_numbers (GtkMenuItem*, gpointer);
  
  // Project dialogs functions
  friend void openldev_project_properties (GtkMenuItem*, gpointer);
  friend void openldev_project_info (GtkMenuItem*, gpointer);
  friend void openldev_add_files (GtkMenuItem*, gpointer);
  friend void openldev_remove_file (GtkMenuItem*, gpointer);
  
  // Build menu commands
  friend void openldev_compile (GtkMenuItem*, gpointer);
  friend void openldev_build (GtkMenuItem*, gpointer);
  friend void openldev_execute (GtkMenuItem*, gpointer);
  friend void openldev_autogen (GtkMenuItem*, gpointer);
  friend void openldev_configure (GtkMenuItem*, gpointer);
  friend void openldev_clean (GtkMenuItem*, gpointer);
  friend void openldev_stop (GtkMenuItem*, gpointer);

  // Help commands
  friend void openldev_help (GtkMenuItem*, gpointer);
  friend void openldev_report_bug (GtkMenuItem*, gpointer);
  friend void openldev_about (GtkMenuItem*, gpointer);

  // Resize signal functions
  friend void on_resize_pane (GtkWidget*, GdkEvent*, gpointer);
  friend void on_resize (GtkWidget*, GdkEvent*, gpointer);

  // GtkTreeView signal functions
  friend bool on_search_box_selected (GtkWidget*, GdkEventButton*, gpointer);
  friend bool on_button_press_event_treeview (GtkWidget*, GdkEventButton*, gpointer);
  friend bool on_build_box_selected (GtkWidget*, GdkEventButton*, gpointer);
  friend void on_status_bar_update (GtkTextBuffer*, GtkTextIter*, GtkTextMark*, gpointer);  

  // GtkButton signal functions
  friend void on_close_toolbars (GtkWidget*, gpointer);
  friend void on_find_next (GtkWidget*, gpointer);
  friend void on_find_and_replace (GtkWidget*, gpointer);
  friend void on_jump_to_line_number (GtkWidget*, gpointer);
  friend bool on_textview_handle_autocomplete(GtkWidget*, GdkEventKey*, gpointer);

private:
  // Class Widgets
  OutputManager *output;
  FindToolbar *findtoolbar;
  GotoLineToolbar *gototoolbar;
  Browser *browser;

  // Container Widgets
  GtkWidget *window;
  GtkWidget *hpaned, *vpaned, *vbox, *hbox;

  // Many types of bars
  GtkWidget *status_bar, *status_bar2;
  GtkWidget *menubar, *toolbar, *menu_source_browser;
  
  // Environment & project settings
  EditorProperties *editor;
  ProjectTemplate *ptemplate;
  TemplateList *list_of_templates;
};

#endif
