/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev_settings.h"

// Load the environment settings dialog and then store the changes that the user
// makes and rewrite the environment settings file. Then update the fonts.
void openldev_environment_settings (GtkMenuItem *menuitem,
                                    gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  EnvironmentDialog *dialog;
  dialog = new EnvironmentDialog(ptr->env);
  int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));
  
  switch (result)
  {
    case GTK_RESPONSE_OK: 
    {
      // Change properties by reference & destroy    
      ptr->env->set_font_face(dialog->get_font_face());
      ptr->env->set_font_size(dialog->get_font_size());
      ptr->env->set_replace_tabs(dialog->get_replace_tabs());
      ptr->env->set_auto_indent(dialog->get_auto_indent());
      ptr->env->set_undo(dialog->get_undo());
      ptr->env->set_wrap(dialog->get_wrap());
      ptr->env->set_wrapsplit(dialog->get_split());
      ptr->env->set_brackets(dialog->get_brackets());
      ptr->env->set_error(dialog->get_error_color());
      ptr->env->set_warning(dialog->get_warning_color());
      ptr->env->set_autosave(static_cast<int>(dialog->get_autosave()));
      ptr->env->set_tabsize(static_cast<int>(dialog->get_tab_size()));
      ptr->env->set_autoload(dialog->get_load_on_launch());
      ptr->env->set_rightmargin(dialog->get_margin());
      ptr->env->write_settings_file();
      
      // Reset fonts for all tabs
      string font_description = ptr->env->get_font_face();
      font_description += " " + ptr->env->get_font_size();
      PangoFontDescription *fd = pango_font_description_from_string(font_description.c_str()); 
      ptr->noteman->noteman_modify_fonts(fd);
      ptr->noteman->noteman_set_properties(true, ptr->env);
      break;
    }
    default:
      break;
  }
  
  gtk_widget_hide(dialog->get_dialog());
  delete dialog;
}

// If a project is open, load the project properties dialog and set the
// changes specified by the user. Also, rewrite the project compile flags to
// the src/Makefile.am
void openldev_project_properties (GtkMenuItem *menuitem,
                                  gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    ProjectPropertiesDialog *dialog;
    dialog = new ProjectPropertiesDialog (ptr->project);
    int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));
    
    if (result == GTK_RESPONSE_OK)
    {
      int begin, end;
      string text, makefile_location, commands;
      
      // Set project properties & save project file
      ptr->project->set_author(dialog->get_author());
      ptr->project->set_email(dialog->get_email());
      ptr->project->set_homepage(dialog->get_homepage());
      ptr->project->set_version(dialog->get_version());
      ptr->project->write_project_file();
      
      // Create a string for the location of the src/Makefile.am
      makefile_location = ptr->project->get_location() + "src/Makefile.am";
      
      // Read in the Makefile text
      text = read_file(makefile_location);
      
      // Find where the compiler flags are & erase them
      begin = text.find("FLAGS =", 0) + 8;
      end = text.find("bin_PROGRAMS", begin) - 1;
      text.erase(begin, end - begin);
      
      // Write first flag without a '\'
      commands = dialog->get_commands();
      text.insert(begin, commands);

      // Write new Makefile.am file
      write_file(makefile_location, text);
    }
    
    gtk_widget_hide(dialog->get_dialog());
    delete dialog;
  }
}

// If a project is open, load the project information dialog.
void openldev_project_info (GtkMenuItem *menuitem,
                            gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->project->get_name().empty()) 
    error("Please open or create\na project first!");
  else 
  {
    ProjectInfoDialog *dialog;
    dialog = new ProjectInfoDialog(ptr->project, ptr->ptemplate);
    int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));
  
    switch (result)
    {
      default:
        break;
    }
  
    gtk_widget_hide(dialog->get_dialog());
    delete dialog;
  }
}
