/***************************************************************************
 *   Copyright (C) 2005 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "printing.h"

void openldev_print_file (GtkMenuItem *menuitem,
                          gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {  
    string file = string("File: ") + ptr->noteman->noteman_get_current_short_filename(),
           font = ptr->env->get_font_face() + string(" Regular ") + ptr->env->get_font_size();
    GnomePrintConfig *cfg = gnome_print_config_default();
    GtkSourcePrintJob *sjob = gtk_source_print_job_new (cfg); 
  
    gtk_source_print_job_set_buffer (sjob, GTK_SOURCE_BUFFER(ptr->noteman->noteman_get_textbuffer()));
    gtk_source_print_job_set_tabs_width (sjob, ptr->env->get_tabsize());
    gtk_source_print_job_set_wrap_mode (sjob, GTK_WRAP_NONE);
    gtk_source_print_job_set_highlight (sjob, false);
    gtk_source_print_job_set_print_numbers (sjob, 5);
    gtk_source_print_job_set_text_margins (sjob, 0.5, 0.5, 0.5, 0.5);
    gtk_source_print_job_set_font (sjob, font.c_str());
    gtk_source_print_job_set_numbers_font (sjob, font.c_str());
  
    GnomePrintJob *job = gtk_source_print_job_print (sjob);
    g_object_unref (G_OBJECT(sjob));
    GtkWidget *dialog = gnome_print_dialog_new (job, (guchar*)"Print Document",
                        GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
  
    gint result = gtk_dialog_run (GTK_DIALOG(dialog));
    gtk_object_destroy (GTK_OBJECT(dialog));
  
    if (result == GNOME_PRINT_DIALOG_RESPONSE_PREVIEW)
      openldev_print_preview (menuitem, data);
    else if (result == GNOME_PRINT_DIALOG_RESPONSE_PRINT)
      gnome_print_job_print (job);
  
    g_object_unref (G_OBJECT(job));
    g_object_unref (G_OBJECT(cfg));
  }
}

void openldev_print_preview (GtkMenuItem *menuitem,
                             gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (ptr->noteman->noteman_get_file_open())
  {  
    string file = string("File: ") + ptr->noteman->noteman_get_current_short_filename(),
           font = ptr->env->get_font_face() + string(" Regular ") + ptr->env->get_font_size();
    GnomePrintConfig *cfg = gnome_print_config_default();
    GtkSourcePrintJob *sjob = gtk_source_print_job_new (cfg); 

    gtk_source_print_job_set_buffer (sjob, GTK_SOURCE_BUFFER(ptr->noteman->noteman_get_textbuffer()));
    gtk_source_print_job_set_tabs_width (sjob, ptr->env->get_tabsize());
    gtk_source_print_job_set_wrap_mode (sjob, GTK_WRAP_CHAR);
    gtk_source_print_job_set_highlight (sjob, false);
    gtk_source_print_job_set_print_numbers (sjob, 5);
    gtk_source_print_job_set_text_margins (sjob, 0.5, 0.5, 0.5, 0.5);
    gtk_source_print_job_set_font (sjob, font.c_str());
    gtk_source_print_job_set_numbers_font (sjob, font.c_str());
  
    GnomePrintJob *job = gtk_source_print_job_print (sjob);
    g_object_unref (G_OBJECT(sjob));
    GtkWidget *preview;

    preview = gnome_print_job_preview_new (job, (guchar*)"Print Preview");
    gtk_widget_show(preview);
  
    g_object_unref (job);
    g_object_unref (cfg);
  }
}
