/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "template.h"

// Load the template file listed at the location received.
void ProjectTemplate::load_template (string file)
{
  string filename = string(INSTALL_PREFIX) + "/share/openldev/templates/" + file + ".template";
  xmlDocPtr doc = xmlParseFile (filename.c_str());
  xmlNodePtr cur;

  if (doc != NULL) 
  {
    cur = xmlDocGetRootElement(doc);
    if (!xmlStrcmp(cur->name, (const xmlChar *) "project"))
    {
      tar = get_xml_attribute("template", doc);
      title = get_xml_attribute("title", doc);
      language = get_xml_attribute("language", doc);
      compile = get_xml_attribute("compile", doc);
      build = get_xml_attribute("build", doc);
      autogen = get_xml_attribute("autogen", doc);
      configure = get_xml_attribute("configure", doc);
      clean = get_xml_attribute("clean", doc);
    }
  }
  
  xmlFreeDoc(doc);
}

string ProjectTemplate::get_xml_attribute (string element, 
                                           xmlDocPtr &doc)
{
  xmlNodePtr cur = xmlDocGetRootElement(doc);
  cur = cur->xmlChildrenNode;
  
  while (cur != NULL)
  {
    if (!xmlStrcmp(cur->name, (const xmlChar *) element.c_str()))
    {
      const char *key = (const char*)xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
      return key;
    }
    
    cur = cur->next;
  }
  
  return "";
}

// Make the compile string replacing {SOURCE} with the filename of the
// file that is to be compiled
string ProjectTemplate::get_compile (string fn)
{
  int location = compile.find("{SOURCE}", 0);
  
  string command = compile;
  command.erase(location, 8);
  command.insert(location, fn);
  
  location = command.find("{OBJECT}", 0);
  command.erase(location, 8);
  fn.erase(fn.find(".", 0), fn.length()-fn.find(".", 0));
  command.insert(location, fn);
  
  return command;
}

string ProjectTemplate::get_language()
{ return language; 
}

string ProjectTemplate::get_build()
{ return build; 
}

string ProjectTemplate::get_autogen()
{ return autogen; 
}

string ProjectTemplate::get_configure()
{ return configure; 
}

string ProjectTemplate::get_clean()
{ return clean; 
}

// Load the templates from the /usr/share/openldev/templates/ folder. The
// index.template file is no longer used, so this function will be renamed
// in the future...
void TemplateList::load_templates()
{
  string directory = string(INSTALL_PREFIX) + "/share/openldev/templates/", name;
  unsigned int l;

  vector<string> entries;
  const char* fn;
  GDir *dir = g_dir_open (directory.c_str(), 0, NULL);
  while ((fn = g_dir_read_name(dir)) != NULL)
    entries.push_back (fn);
  
  for (l=0; l < entries.size(); l++)
    if ((entries.at(l).find(".template", 0) != string::npos) && (entries.at(l) != "index.template"))
      file.push_back(entries.at(l).substr(0, entries.at(l).find(".template", 0)));

  for (l=0; l < file.size(); l++)
  {
    string str = read_file(directory + file.at(l) + ".template");
    str.erase(0, str.find("<title>", 0) + 7);
    str.erase(str.find("</title>", 0), str.length() - str.find("</title>", 0));
    title.push_back(str);
  }
}
