/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "treeview.h"

bool is_error_or_warning (string output);

void OpenLDev::openldev_initialize_treeview()
{
  gtk_widget_set_size_request(browser->get_files_swin(), env->get_treeview_width(), 0);
  gtk_widget_set_size_request(output->get_terminal(), 0, env->get_buildbox_height());
  browser->populate(project);
}

// Open file selected by the tree view. This function gets the filename
// from the project files TreeView and loads it if the file selected is not
// one of the headers
void OpenLDev::openldev_handle_treeview_selection()
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(browser->get_files()));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
  {
    gchar *fn;
    gtk_tree_model_get(model, &iter, 0, &fn, -1);
   
    // If the text is not a group title, open it
    string file = project->get_location() + "src/" + fn;
    openldev_open_file(file);
  }
}

// Add new file(s) to the project. The user can choose one or more files from
// the FileChooserDialog. They are then parsed and added to the project one
// at a time. The project settings file and the src/Makefile.am are then
// rewritten.
void openldev_add_files (GtkMenuItem *menuitem,
                         gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  GtkWidget *dialog;
  dialog = gtk_file_chooser_dialog_new ("Choose File", 
                                        GTK_WINDOW(ptr->window), 
                                        GTK_FILE_CHOOSER_ACTION_OPEN,
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_ADD, GTK_RESPONSE_OK,
                                        NULL);
  string folder = string(ptr->project->get_location() + "src/");
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), folder.c_str());
  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), true);
  
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  switch (result)
  {
    // If OK is pressed
    case(GTK_RESPONSE_OK):
    {
      int location;
      GSList *fns = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER(dialog));
      
      // Parse the files & add them to the project
      for (unsigned int i=0; i < g_slist_length(fns); i++)
      {
        string current = (const char*) g_slist_nth_data (fns, i);
        location = current.find_last_of("/");
        while (location != (signed int)string::npos)
        {
          current.erase(0, location+1);
          location = current.find_last_of("/");
        }
        current += "/";
        
        ptr->project->add_file(current);
      }
      
      // Create the new project file & src/Makefile.am
      ptr->openldev_populate_treeview();
      ptr->project->write_project_file();
      ptr->project->write_source_makefile();
      
      string fns2 = ptr->project->get_source_files() + ptr->project->get_header_files(),
             loc = ptr->project->get_location() + string("src/");
      ptr->noteman->noteman_autolist_clean_all ();
      ptr->noteman->noteman_autolist_parse_files (fns2, loc);
      break;
    }
    default:
      break; 
  }

  gtk_widget_hide (dialog);
}

// Remove the currently selected file from the list. The project settings file 
// and the src/Makefile.am are then rewritten.
void openldev_remove_file (GtkMenuItem *menuitem,
                           gpointer data)
{  
  OpenLDev *ptr = (OpenLDev*) data;
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ptr->browser->get_files()));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
  {
    gchar *fn;
    gtk_tree_model_get(model, &iter, 0, &fn, -1);
    ptr->project->remove_file(fn);
  }
  
  // Refresh the list and rewrite the project file and src/Makefile.am
  ptr->openldev_populate_treeview();
  ptr->project->write_project_file();
  ptr->project->write_source_makefile();
}

// Populate or update the TreeView. The sources, headers and other files are
// parsed and added one at a time to their corresponding section in the project
// files TreeView. The branches are then expanded and the first item is selected
void OpenLDev::openldev_populate_treeview()
{
  browser->populate(project);
}

// Catch button press event & open popup menu or handle selection. This function
// receives the button event. If it is a right click, then the popup menu is
// shown. If it is a double click, the selected file is opened
bool on_button_press_event_treeview (GtkWidget *widget,
                                     GdkEventButton *event,
                                     gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU(ptr->menu_source_browser), 
                    NULL, NULL, NULL, data, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
    ptr->openldev_handle_treeview_selection();
  else
    return false;
  
  return true;
}

// Catch the build box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and check if
// it is an error or warning. If it is, then the file containing the error or
// warning is opened. The cursor is then moved line of the error or warning
bool on_build_box_selected (GtkWidget *widget, 
                            GdkEventButton *event, 
                            gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      string file;
      gchar *output;
      int linenumber;
      gtk_tree_model_get(model, &iter, 0, &output, -1);
      
      // It the output is an error or a warning..
      if (is_error_or_warning(output))
      {
        file = string(output);
        linenumber = get_file_name_and_line(file) - 1;
        ptr->openldev_open_file(ptr->project->get_location() + "src/" + file);
        
        // Move the cursor to the line of the error & scroll to the cursor
        GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman->noteman_get_textbuffer());
        GtkTextIter iter;

        gtk_text_buffer_get_iter_at_line(buffer, &iter, linenumber);
        gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_insert(buffer), &iter);
        gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
        ptr->noteman->noteman_scroll_to_mark();
        gtk_widget_grab_focus(ptr->noteman->noteman_get_textview());
      }
    }
    
    return true;
  }
  
  return false;
}

// Check if the output is an error or warning. If it only has two colons in
// close proximity of eachother in the beginning, then it is an error
// or a warning.
bool is_error_or_warning (string output)
{
  // Find :'s (It is an error or warning if 2 :'s are found close together
  int location  = output.find(":", 0);
  int location2 = output.find(":", location+1);
  
  // Check if :'s were found & return if they were in proper format
  if (location != (signed int)string::npos)
    if (location2 != (signed int)string::npos && (location2-location < 5))
      return true;
  
  return false;
}
